#-*-coding:utf-8-*-
"""
    Objet package représente un paquet
    Préféré installer.WebAppsInstaller
"""
from os.path import isfile, isdir
from tempfile import mktemp

from twisted.python import log

from ead2.config.config import dico as dico_creole
from ead2.backend.actions.tools import command_statusoutput as command
from ead2.backend.actions.lib.error import exception

from ead2.backend.actions.lib.envole.install.error import (NotInstalled,
                                    NotCreoleActive, NoLogFileInMemory,
                                    NoLogFile, DisabledApp, DpkgError)
from ead2.backend.actions.lib.envole.install.cmds import (
                                    DPKG_VERSION, APT_INSTALL,
                                    DPKG_STATUS)

from ead2.backend.actions.lib.envole.install.status import STATUS_DICT

def tail(filename):
    """
        Renvoie le contenu courant du fichier filename
    """
    return file(filename).read()

class BasePackage:
    """
        Interface de manipulation d'un paquet d'application web
        :name: nom du paquet
        :version: version disponible du paquet
        :creole_key: clé creole pour l'activation de l'application
        :default_active: doit-on considéré que cette application
                         est active par défaut
    """
    def __init__(self, name, version, creole_key=None, default_active="oui"):
        self.name = name
        self.creole_key = creole_key or name
        self.default_active = default_active
        self.logfile = None
        self.version = version
        # tag ajouté lorsque l'application n'est pas installable
        # (signalé depuis un fichier plat)
        self.is_disabled = False
        # Cache prévu pour éviter de lancer des commandes
        # dpkg-query toutes les deux secondes
        self.cache = dict()
        self.renew_cache()

    def renew_cache(self):
        """
            Rafraichit le cache pour l'application en cours
        """
        self.cache['version'] = self.__version()
        self.cache['status'] = self.__status()

    def __version(self):
        """
            Lance un dpkg-query pour récupérer
            la version installée du paquet
        """
        cmd = DPKG_VERSION % dict(name=self.name)
        ret_code, ret_value = command(cmd)
        if ret_code:
            if "Aucun paquet" in ret_value:
                return None
            else:
                log.err(ret_value)
                raise DpkgError("Une erreur est survenue à la récupération \
de la version de l'application, contactez votre administrateur.")
        else:
            lines = ret_value.splitlines()
            if len(lines) >= 1:
                return lines[0].strip()
            else:
                log.err("Le paquet '%s' n'a pas de version (il a du être \
désinstallé à la main)" % (self.name,))
                return None

    def __status(self):
        """
            Lance un dpkg query pour renvoyer le statut du paquet
        """
        cmd = DPKG_STATUS % dict(name=self.name)
        ret_code, ret_value = command(cmd)
        if ret_code:
            if "Aucun paquet" in ret_value:
                return 0
            else:
                log.err(ret_value)
                raise DpkgError("Une erreur est survenue à la récupération \
du statut d'installation de l'application, contactez votre administrateur.")
        else:
            lines = ret_value.splitlines()
            if len(lines) >= 1:
                return STATUS_DICT.get(lines[0].strip(),
                                       STATUS_DICT['default'])
            else:
                return 0

    def disable(self):
        """
            Désactive une application
        """
        self.is_disabled = True

    def active(self):
        """
            Teste l'activation de l'application
        """
        return dico_creole.get(self.creole_key, self.default_active) == 'oui'

    def is_installed(self):
        """
            Renvoie True si le paquet est installé
        """
        return (self.__version() is not None) or self.__status() == 2

    def update(self):
        """
            Update le paquet
        """
        if self.is_disabled:
            raise DisabledApp("L'application a été désactivée par \
l'administrateur.")
        if not self.is_installed():
            raise NotInstalled("L'application n'a pas encore été installée.")
        elif not self.active():
            raise NotCreoleActive("L'application a été désactivée par \
l'administrateur.")
        else:
            return self.install()

    def __build_install_cmd(self):
        """
            Renvoie la commande à lancer pour installer le paquet
        """
        opt_dict = {'name':self.name, 'logfile':self.logfile}
        return APT_INSTALL % opt_dict

    def __gen_logfile(self):
        """
            Génère le nom d'un fichier de log pour l'installation du paquet
        """
        while True:
            logfile = mktemp(".log", self.name, '/var/log/ead')
            if not isfile(logfile) and isdir(logfile):
                break
        return logfile

    def install(self):
        """
            Installe le paquet
        """
        if self.is_disabled:
            raise DisabledApp("L'application a été désactivée par \
l'administrateur.")
        if self.is_installed() and not self.active():
            raise NotCreoleActive("L'application a été désactivée par " + \
                    "l'administrateur : impossible de la mettre à jour.")
        self.logfile = self.__gen_logfile()
        ret_code, ret_value = command(self.__build_install_cmd(),
                                      "at now")
        if ret_code:
            log.err(ret_value)
            exception("Une erreur est survenue au lancement de " + \
                      "l'installation de '%s'" % (self.name,))
        else:
            return self.logfile

    def tail(self):
        """
            Renvoie le contenu du logfile
        """
        if not self.logfile:
            log.err("On tente de récupérer les logs d'installation de \
'%s' mais on a pas de nom de fichier." % (self.name,))
            raise NoLogFileInMemory("Un erreur est survenue à la \
récupération des informations d'avancement, contactez l'administrateur")
        elif not isfile(self.logfile):
            log.err("On tente de récupérer les logs d'installation de \
'%s' mais le fichier '%s' n'existe pas." % (self.name, self.logfile))
            raise NoLogFile("Un erreur est survenue à la récupération \
des informations d'avancement, contactez l'administrateur")
        else:
            return tail(self.logfile)

class Package(BasePackage):
    """
        Couche rendu d'un paquet
        :name: nom du paquet
        :version: version disponible du paquet
        :creole_key: clé creole pour l'activation de l'application
        :default_active: doit-on considéré que cette application est
                         active par défaut
        :description: descriptif pour la mise en page
    """

    def __init__(self, name, version, creole_key, default_active, description=''):
        BasePackage.__init__(self, name, version, creole_key, default_active)
        self.description = description

    def get_actual_status(self):
        """
            Renvoie une phrase pour décrire le status d'installation du paquet
            - 0 : aucune action n'est possible
            - 1 : il est possible d'installer l'application
            - 2 : il est possible de réinstaller l'application
        """
        status = self.cache['status']
        version = self.cache['version']
        active = self.active()
        if status == 2:
            if active:
                return 2, "L'application a été désinstallée manuellement."
            else:
                return 0, "Votre administrateur a désactivé l'application."
        elif status == 0:
            if active:
                return 1, "L'application n'est pas encore installée."
            else:
                return 0, "Votre administrateur a désactivé l'application."
        elif status == 1:
            if version == self.version:
                if active:
                    return 0, "La version la plus récente est déjà installée."
                else:
                    return 0, "Votre administrateur a désactivé l'application."
            elif version <= self.version:
                if active:
                    return 2, "La version %s est actuellement installée." % (
                                                                    version,)
                else:
                    return 0, "Votre administrateur a désactivé l'application."
            else:
                return 0, "Une version plus récente (%s) est déjà installée \
(Signalez le à votre administrateur)" % version
        else:
            return 0, "Le statut de votre application est inconnu, impossible \
de l'installer."

    def repr(self):
        """
            Renvoie un dictionnaire décrivant le paquet
        """
        action_num, message = self.get_actual_status()
        return dict(action_num=action_num,
                    message=message,
                    name=self.name,
                    description=self.description,
                    version=self.version
                    )
