#-*-coding:utf-8-*-
"""
    Action de gestion d'Envole2.0
    Syncrhonise les groupes dans les réseaux utilisateurs.
"""

from twisted.python import log
from twisted.internet.defer import Deferred, DeferredList

from scribe.eolegroup import Group

from posh.api.scribe_api import EnvoleUserGroups

from ead2.backend.lib.action import Dict
from ead2.backend.actions.lib.main import Main
from ead2.backend.actions.lib.error import _clean_errors, exception
from ead2.backend.actions.lib.widgets import ajax, main as M, form as F
from ead2.backend.actions.tools import format_form_result

GROUP_TYPES = ('Niveau', 'Classe', 'Matiere', 'Equipe',
               'Option', 'Service', 'Administrateurs', 'Base')
LIBELLES = {'Matiere':"Matière",
            "Equipe":"Équipes pédagogiques",
            "Base": "Groupes métiers"}

ENTETE = "Cet outil vous permet de synchroniser un (des) groupe(s) et ses (leurs) utilisateurs dans les bases de données de Envole2.0, pour les rendre disponibles dans les groupes du portail (réseau social de l'établissement)."

def get_choosetype_btn(group_type, server_nb, actionname):
    """
        Crée un bouton pour synchroniser les groupes de type group_type
    """
    href = ajax.call(server_nb, actionname,
                     group_type=group_type,
                     container='div_group_list')
    libelle = "%s" % LIBELLES.get(group_type, group_type)
    return M.Lien(href=href, libelle=libelle)

def get_synchrogroup_link(groupname, server_nb, actionname):
    """
        Renvoie le lien pour synchroniser un groupe dans envole
    """
    href = ajax.call(server_nb, actionname,
                     synchrogroup=groupname,
                     container='div_synchro_message')
    return M.Lien(href, libelle=groupname)

def get_synchrogroup_checkbox(groupname):
    """
        Renvoie la checbox pour la synchro des groupes
    """
    return F.Checkbox(name=groupname,
                      checked=True,inline=True)


def get_synchrogroup_row(groupname, server_nb, actionname):
    """
        renvoie une ligne représentant un groupe
    """
    return dict(synchrogroup_link=get_synchrogroup_link(groupname,
                                                        server_nb,
                                                        actionname),
                synchrogroup_checkbox=get_synchrogroup_checkbox(groupname)
                )

def synchronize_group(groupname):
    """
        Synchronise le groupe groupname
    """
    posh_api = EnvoleUserGroups()
    posh_api.synchronize_usergroup(groupname, force=True)
    return dict(message = "Le groupe %s a bien été synchronisé" % groupname)

def synchronize_groups(group_list):
    """
        Synchronise les groupes de la liste
    """
    log.msg(" + Synchronisation des groupes : %s" % (group_list,))
    posh_api = EnvoleUserGroups()
    posh_api.scribegroup.ldap_admin.connect()
    deferreds = list()
    for grname in group_list:
        deferred = Deferred()
        deferred.callback(grname)
        deferred.addCallback(posh_api._synchronize_usergroup, force=True)
        deferreds.append(deferred)
    deferredlist = DeferredList(deferreds)
    deferredlist.addCallback(synchronize_success, posh_api=posh_api)
    return deferredlist

def synchronize_success(result, posh_api=None):
    """
        Gère la réussite de la synchronisation
    """
    posh_api.scribegroup.ldap_admin.close()
    posh_api.save_usergroupids()
    error_tag = False
    error_message = " - Une erreur est survenue lors de la \
synchronisation des groupes : \\n"
    for success, returned_value in result:
        if not success:
            message = returned_value.getErrorMessage()
            error_message += " * %s\\n" % (message,)
            error_tag = True
    if error_tag:
        exception(error_message)
    return dict(message="Les groupes sélectionnés ont bien été synchronisé.")

def get_grouplist(group_type):
    """
        Renvoie la liste formattée des group de type group_type
    """
    eolegroup = Group()
    eolegroup.ldap_admin.connect()
    for group in eolegroup._get_groups(group_type):
        yield group
    eolegroup.ldap_admin.close()

class EnvoleSynchronize(Main):
    """
        Outil de synchronisation
    """
    name = "envole_synchronize"
    category = "Envole"
    libelle = "Synchronisation"
    description = "Synchronisation des groupes et utilisateurs dans \
Envole 2.0"
    # templates utilisés
    widget_templates = ['main', 'form', 'formselect']
    template = 'envole_synchronize'

    # entrées de requête
    request = Dict(default={},
            doc="arguments de la requete en cours cote frontend",
            keys=['server', 'action', 'group_type', 'synchrogroup'])
    form_result = Dict(default={},
                       doc="",
                       keys=['synchro_groups'])

    def execute(self):
        """
            Répond aux appels du frontend
        """
        self.parse_request()
        if not set(['group_type', 'synchrogroup']).intersection(set(self.params.keys())) and \
                                              not 'synchro_groups' in self.form_result.keys():
            log.msg(" + Premier passage")
            return_datas = dict(titre=self.description,
                                entete="%s<br/><br /><h2>Lancer \
la synchronisation des groupes de type :</h2>" % (ENTETE))
            deferred = Deferred()
            deferred.callback(return_datas)
            deferred.addCallback(self.get_btns)
            deferred.addErrback(_clean_errors,
                    "Erreur dans la présentation de l'action")
        elif self.params.get('group_type'):
            group_type = self.params.get('group_type')[0]
            log.msg(" + On liste les groupes de type : %s" % (group_type,))
            subtitle = "Groupes de type : %s" % (group_type,)
            deferred = Deferred()
            deferred.callback(group_type)
            deferred.addCallback(get_grouplist)
            deferred.addErrback(_clean_errors,
                    "Erreur à la récupération des la liste des groupes")
            deferred.addCallback(self.format_group_list, subtitle=subtitle)
            deferred.addErrback(_clean_errors,
                    "Erreur au formattage de la liste des groupes")
            deferred.addCallback(self.add_bouton_valid)
            deferred.addErrback(_clean_errors,
                    "Erreur à la génération du bouton valider")
        elif self.params.has_key('synchrogroup'):
            #FIXME:on synchro le groupe (on teste son existence ou pas ?)
            deferred = Deferred()
            groupname = self.params.pop('synchrogroup')[0]
            deferred.callback(groupname)
            deferred.addCallback(synchronize_group)
            deferred.addErrback(_clean_errors,
              "Erreur à la synchronisation du groupe : %s" % (groupname,))
        elif self.form_result.has_key('synchro_groups'):
            #FIXME:on synchro les groupes (on teste son existence ou pas ?)
            deferred = Deferred()
            formulaire = format_form_result(self.form_result.pop('synchro_groups'))
            deferred.callback(formulaire.keys())
            deferred.addCallback(synchronize_groups)
            deferred.addErrback(_clean_errors,
               "Erreur à la synchronisation des groupes")
        else:
            pass
            #FIXME:ya un truc à tester ici
        deferred.addCallback(self.main)
        deferred.addErrback(self.send_error)
        return deferred

    def main(self, return_dict):
        """
            Renvoie les informations
        """
        return self.send_all(return_dict,
                             template=self.template,
                             templates=self.widget_templates)

    def send_error(self, error, message=None):
        """
            Lancé en cas d'erreur dans l'éxécution
        """
        log.msg(" - Dans send_error -")
        if not message:
            message = error.getErrorMessage()
            log.msg(error.getErrorMessage())
        return_dict = dict(message=message)
        return self.send_all(return_dict,
                             template=self.template,
                             templates=self.widget_templates)

    def get_btns(self, return_dict):
        """
            Place les boutons dans le dictionnare return_dict
        """
        return_dict['boutons'] = [get_choosetype_btn(group_type,
                                                     self.server_nb,
                                                     self.name)
                                      for group_type in GROUP_TYPES]
        return return_dict

    def format_group_list(self, group_list_generator, subtitle="Aucun"):
        """
            Renvoie la liste des groupes formattés
        """
        return_datas = {}
        for groupname in group_list_generator:
            return_datas.setdefault('group_rows', []).append(
                                    get_synchrogroup_row(groupname,
                                                    self.server_nb,
                                                         self.name)
                                                    )
        if not 'group_rows' in return_datas.keys():
            return_datas['group_rows'] = "Il n'existe aucun groupe de ce type."
        return_datas['subtitle'] = subtitle
        return return_datas

    def add_bouton_valid(self, return_datas):
        """
            Rajoute le bouton valider dans le dictionnaire renvoyé
        """
        href = ajax.valid(self.server_nb, self.name,
                          ['synchro_groups'],
                          container='div_synchro_message')
        return_datas['bouton_valid'] = M.Submit(href=href,
                title="Synchroniser les groupes sélectionnés.")
        return return_datas
