#!/usr/bin/python
#-*-coding:utf-8-*-
"""
    Script pour le lancement de l'installation d'un paquet
    Lance :
    Query-Auto
    apt-get install --download-only
    apt-get install
    Peut être utilisé en ligne de commande ou depuis l'ead (voir DOC)
"""
DOC = """
Usage : envole_install.py [OPTION] [PACKAGENAME]

Obligatoire:
-l --logfile : chemin vers le fichier de log utilisé
-v --version : version du paquet à installer

Optionnel:
-D             : paquet de dev
-C             : paquet candidat
--ead          : appel depuis l'ead (pour les logs)

Ex:
    1) envole_install.py -l /tmp/ajaxplorer.log -v 2.5.4-eole18 ajaxplorer
    Installe l'application ajaxplorer en version 2.5.4-eole18 et log les informations dans /tmp/ajaxplorer.log

    2) envole_install.py -l /tmp/ajaxplorer.log -u -v 2.5.4-eole18
    Update l'application ajaxplorer en version 2.5.4-eole18 et log les informations dans /tmp/ajaxplorer.log
"""

import getopt, sys

from pyeole.log import make_logger
from creole import eolelock

sys.path.append('/usr/share')
from ead2.backend.actions.tools import command_statusoutput
from ead2.backend.actions.lib.thread.logger import Logger
from ead2.backend.actions.lib.envole.install.cmds import (
                    QUERY_AUTO, QUERY_AUTO_D, QUERY_AUTO_C,
                    APT_DOWNLOAD, APT_INSTALL)

LOGFMT = "%(asctime)s - %(levelname)s %(message)s"
DATEFMT = '%Y-%m-%d %H:%M:%S'
EADLOG = make_logger('ead2.backend.bin.envole_install', '/var/log/ead/ead-server.log',
                  loglevel='DEBUG', logfmt=LOGFMT, datefmt=DATEFMT)


class Locker:
    """
        Interface de gestion des locks (compatible avec l'outil de log)
    """
    @staticmethod
    def add_lock():
        """
            Ajoute un lock
        """
        eolelock.add_lock('eolemaj-auto')

    @staticmethod
    def del_lock():
        """
            Supprime le lock
        """
        eolelock.del_lock('eolemaj-auto')

    @staticmethod
    def check_locks():
        """
            Teste l'existence de lock sur le serveur
        """
        if eolelock.check_all_locks():
            return True
        else:
            return False

def usage():
    """
        print la doc
    """
    print DOC

def main():
    """
        lance la commande
    """
    try:
        opts, args = getopt.getopt(sys.argv[1:],
                "l:v:h:C:D",
                 ["logfile","version","help", "ead"])
    except getopt.GetoptError, err:
        print(err)
        usage()
        sys.exit(2)

    logfile = None
    version = None
    mode = None
    ead = False
    for key, value in opts:
        if key in ('-l', '--logfile'):
            logfile = value
        elif key in ('-v', '--version'):
            version = value
        elif key in ('-h', '--help'):
            usage()
            sys.exit(0)
        elif key == '-D':
            mode = 'dev'
        elif key == '-C':
            mode = 'candidat'
        elif key == '--ead':
            ead = True
        else:
            print("Argument innatendue")
            usage()
            sys.exit(2)
    if logfile == None or version == None:
        print("IL MANQUE DES ARGUMENTS")
        usage()
        sys.exit(2)
    if len(args) < 1:
        print("IL MANQUE LE NOM DU PAQUET")
        usage()
        sys.exit(2)
    else:
        packagename = args[0]
    if ead:
        EADLOG.info("Lancement de l'installation de '%s' en version '%s'" % (
                                                    packagename, version))
    Installer(logfile, packagename, version, mode, ead).process()

class Installer(Logger):
    """
        Manager pour l'installation d'application
    """
    def __init__(self, logfile, app_name, version, mode=None, ead=False):
        lockmanager = Locker()
        self.ead = ead
        if self.ead:
            super(Installer, self).__init__(logfile, lockmanager=lockmanager,
                                                          root_logger=EADLOG)
        else:
            super(Installer, self).__init__(logfile, lockmanager=lockmanager)
        self.log("<h3># Installation de l'application : '%s' #</h3>" % app_name)
        self.log("<hr>")
        self.version = version
        self.app_name = app_name
        self.mode = mode

    def process(self):
        """
            Procède à l'installation du paquet
        """
        if self.lockmanager.check_locks():
            if self.ead:
                EADLOG.error("Un verrour existe, l'opération doit être abandonnée.")
            self.log_err("!!! Un verrou existe, l'opération a due être \
abandonnée : contactez votre administrateur !!!")
        try:
            self.__query_auto()
            # On remet le lock que Query-Auto vient d'enlever
            self.lockmanager.add_lock()
            self.__download()
            self.__install()
            self.log_end(unlock=True)
            sys.exit(0)
        except Exception:
            self.lockmanager.del_lock()
            if self.ead:
                EADLOG.exception("Une erreur est survenue")
            self.log_err("!!! Une erreur inconnue est survenue : contactez \
votre administrateur !!!")
            sys.exit(1)

    def __query_auto(self):
        """
            Lance la commande query_auto
        """
        self.log("Étape 1 : Récupération de la liste des applications/paquets disponibles ...")
        if self.ead:
            EADLOG.info(" * Lancement du Query-Auto")
        if self.mode is None:
            ret_code, ret_value = command_statusoutput(QUERY_AUTO)
        elif self.mode == 'candidat':
            print("Paquet candidat")
            ret_code, ret_value = command_statusoutput("echo 'O'", QUERY_AUTO_C)
        elif self.mode == 'dev':
            print("Paquet de dev")
            ret_code, ret_value = command_statusoutput("echo 'O'", QUERY_AUTO_D)
        if ret_code:
            if self.ead:
                EADLOG.error(ret_value)
            else:
                self.log(ret_value)
            self.log_err("Une erreur est survenue à la récupération des \
informations")
        else:
            if self.ead:
                EADLOG.info(ret_value)
            else:
                self.log(ret_value)
            self.log_ok()

    def __download(self):
        """
            Lance la commande apt-eole install paquet=version --download-only
        """
        self.log("Étape 2 : Téléchargement de l'application ...")
        if self.ead:
            EADLOG.info(" * Téléchargement du paquet")
        ret_code, ret_value = command_statusoutput(APT_DOWNLOAD % (
                                      self.app_name, self.version))
        if ret_code:
            if self.ead:
                EADLOG.error(ret_value)
            else:
                self.log(ret_value)
            self.log_err("!! Une erreur est survenue au téléchargement de \
l'application !!")
        else:
            if self.ead:
                EADLOG.info(ret_value)
            else:
                self.log(ret_value)
            self.log_ok()

    def __install(self):
        """
            Lance l'installation du paquet apt-eole install paquet=version
        """
        if self.ead:
            EADLOG.info(" * Installation de l'application")
        self.log("Étape 3 : Installation de l'application ...")
        ret_code, ret_value = command_statusoutput(APT_INSTALL % (
                                     self.app_name, self.version))
        if ret_code:
            if self.ead:
                EADLOG.error(ret_value)
            else:
                self.log(ret_value)
            self.log_err("!! Une erreur est survenue à l'installation de \
l'application !!")
        else:
            if self.ead:
                EADLOG.info(ret_value)
            else:
                self.log(ret_value)
            self.log_ok()
            self.log("<br /><h2> L'application '%s' est installée et est \
désormais disponible</h2>" % self.app_name)

if __name__ == '__main__':
    main()
